import java.io.*;
import java.util.*;

/**
 * Verifies that samples in the problem statements correspond to the appropriate text files.
 *
 * @author Roman Elizarov
 */
public class VerifySamples {
    private static final String SEP = File.separator;
    private static final String PATH_SEP = File.pathSeparator;
    private static final String TEXTS_DIR = "../statements/texts/".replace("/", SEP);
    private static final String PROBLEMS_DIR = "../problems/".replace("/", SEP);
    private static final String PROBLEM_PROPS = "problem.properties";
    private static final String TESTS_DIR = "/tests/".replace("/", SEP);
    private static final String TEXT_EXT = ".tex";
    private static final String ANS_EXT = ".a";
    private static final String OUT_EXT = ".out";

    private static final String CHECK_JAR = "check.jar";
    private static final String TESTLIB4J = "../testlib/testlib4j.jar".replace("/", SEP);
    private static final String CHECK_CMD = "{java} -mx256m -ss64m -cp {check}" + PATH_SEP + "{testlib4j} " +
            "-ea ru.ifmo.testlib.CheckerFramework Check {inf} {out} {ans}";

    private static final String EXMP_INPUT = "\\exmp{";
    private static final String EXMP_OUTPUT = "}{";
    private static final String EXMP_END = "}%";

    private static final int MODE_TEXT = 0;
    private static final int MODE_INPUT = 1;
    private static final int MODE_OUTPUT = 2;

    private enum CmpRes {FAILED, WRONG_IN, REJECTED_OUT, OTHER_OUT, OK}

    private static void copyContent(InputStream in, OutputStream out) {
        byte[] buf = new byte[4096];
        int len;
        try {
            while ((len = in.read(buf)) > 0)
                out.write(buf, 0, len);
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private static class Redirector extends Thread {
        private final InputStream in;
        private final ByteArrayOutputStream out = new ByteArrayOutputStream();

        Redirector(InputStream in) {
            this.in = in;
        }

        public void run() {
            copyContent(in, out);
        }
    }

    public static void main(String[] args) {
        String[] list = new File(TEXTS_DIR).list();
        ArrayList<String> names = new ArrayList<>();
        if (list != null)
            for (String s : list) {
                String name = new File(s).getName();
                if (name.endsWith(TEXT_EXT)) {
                    names.add(s.substring(0, name.length() - TEXT_EXT.length()));
                }
            }
        if (names.isEmpty()) {
            System.err.println("NO TEXTS WERE FOUND.");
            return;
        }
        System.out.println(names.size() + " text(s) were found.");
        Collections.sort(names);
        for (String s : names)
            verify(s);
    }

    private static void verify(String name) {
        System.out.print("====== Verifying " + padR(name, 10) + " - ");

        Properties problemProps;
        try {
            problemProps = readProperties(PROBLEMS_DIR + name + SEP + PROBLEM_PROPS);
        } catch (IOException e) {
            System.err.println();
            System.err.println("PROBLEM PROPERTIES NOT FOUND " + e.getMessage());
            return;
        }
        String tests = problemProps.getProperty("tests", "##");

        int n = 0;
        CmpRes cmpres = CmpRes.OK;
        try {
            try (BufferedReader in = new BufferedReader(new FileReader(TEXTS_DIR + name + TEXT_EXT))) {
                String line;
                int mode = 0;
                ArrayList<String> input = new ArrayList<>();
                ArrayList<String> output = new ArrayList<>();
                while ((line = in.readLine()) != null) {
                    switch (mode) {
                        case MODE_TEXT:
                            if (line.trim().equals(EXMP_INPUT))
                                mode = MODE_INPUT;
                            break;
                        case MODE_INPUT:
                            if (line.trim().equals(EXMP_OUTPUT))
                                mode = MODE_OUTPUT;
                            else
                                input.add(unescape(line));
                            break;
                        case MODE_OUTPUT:
                            if (line.trim().equals(EXMP_END)) {
                                mode = MODE_TEXT;
                                n++;
                                cmpres = CmpRes.values()[Math.min(
                                        cmpres.ordinal(),
                                        compare(name, formatTestNo(tests, n), input, output).ordinal())];
                                input.clear();
                                output.clear();
                            } else
                                output.add(unescape(line));
                            break;
                    }

                }
            }
        } catch (IOException e) {
            System.out.println(" - ERROR " + e);
            return;
        }
        System.out.println(n + " example(s) " + cmpres);
    }

    private static String formatTestNo(String tests, int n) {
        int i = tests.indexOf('#');
        int j = tests.lastIndexOf('#');
        String fmt = tests.substring(0, i) + "%0" + (j - i + 1) + "d" + tests.substring(j + 1);
        return String.format(fmt, n);
    }

    private static CmpRes compare(String name, String no, ArrayList<String> input, ArrayList<String> output) {
        String prob_dir = PROBLEMS_DIR + name;
        String inf_name = prob_dir + TESTS_DIR + no;
        String ans_name = inf_name + ANS_EXT;

        if (!compareFile(inf_name, input, true))
            return CmpRes.WRONG_IN;

        String out_name = prob_dir + SEP + name + OUT_EXT;
        if (!saveFile(out_name, output))
            return CmpRes.FAILED;

        Map<String, String> props = new HashMap<>();
        props.put("java", System.getProperty("java.home") + SEP + "bin" + SEP + "java");
        try {
            props.put("check", getAbsFileAndEnsureExists(prob_dir + SEP + CHECK_JAR));
            props.put("testlib4j", getAbsFileAndEnsureExists(TESTLIB4J));
            props.put("inf", getAbsFileAndEnsureExists(inf_name));
            props.put("out", getAbsFileAndEnsureExists(out_name));
            props.put("ans", getAbsFileAndEnsureExists(ans_name));
        } catch (FileNotFoundException e) {
            System.err.println();
            System.err.println("FILE NOT FOUND " + e.getMessage());
            return CmpRes.FAILED;
        }

        ProcessBuilder pb = new ProcessBuilder();
        pb.directory(new File(prob_dir + SEP));
        pb.command(buildCommand(CHECK_CMD, props));
        pb.redirectErrorStream(true);
        int checkCode;
        Redirector redirector;
        try {
            Process p = pb.start();
            redirector = new Redirector(p.getInputStream());
            redirector.start();
            checkCode = p.waitFor();
        } catch (Exception e) {
            System.err.println();
            System.err.println("CANNOT RUN " + pb.command() + " - ERROR " + e);
            return CmpRes.FAILED;
        }
        if (checkCode != 0) {
            System.err.println();
            copyContent(new ByteArrayInputStream(redirector.out.toByteArray()), System.err);
            System.err.println("CHECK REJECTED OUTPUT FOR TEST " + no);
            return CmpRes.REJECTED_OUT;
        }

        if (!compareFile(ans_name, output, false))
            return CmpRes.OTHER_OUT;

        return CmpRes.OK;
    }

    private static Properties readProperties(String fileName) throws IOException {
        Properties props = new Properties();
        try (InputStream in = new FileInputStream(fileName)) {
            props.load(in);
        }
        return props;
    }

    private static String[] buildCommand(String cmd, Map<String, String> props) {
        String[] res = cmd.split("\\s+");
        for (int i = 0; i < res.length; i++)
            for (Map.Entry<String, String> entry : props.entrySet()) {
                String key = entry.getKey();
                String value = entry.getValue();
                res[i] = res[i].replace("{" + key + "}", value);
            }
        return res;
    }

    private static String getAbsFileAndEnsureExists(String name) throws FileNotFoundException {
        File file = new File(name).getAbsoluteFile();
        if (!file.exists())
            throw new FileNotFoundException(file.toString());
        return file.toString();
    }

    private static boolean saveFile(String fileName, ArrayList<String> lines) {
        try {
            PrintWriter out = new PrintWriter(new FileWriter(fileName));
            for (String s : lines)
                out.println(s);
            out.close();
        } catch (IOException e) {
            System.err.println();
            System.err.println("CANNOT WRITE " + fileName + " - ERROR " + e);
            return false;
        }
        return true;
    }

    private static boolean compareFile(String fileName, ArrayList<String> lines, boolean verbose) {
        try {
            try (BufferedReader in = new BufferedReader(new FileReader(fileName))) {
                String line;
                int lineNo = 0;
                while ((line = in.readLine()) != null) {
                    if (lineNo >= lines.size())
                        lines.add("");
                    String expect = lines.get(lineNo);
                    boolean lineOk = line.equals(expect);
                    lineNo++;
                    if (!lineOk) {
                        if (verbose) {
                            System.err.println();
                            System.err.println("MISMATCH IN FILE " + fileName + " ON LINE " + lineNo);
                            System.err.println(" - EXPECTED: " + expect);
                            System.err.println(" -    FOUND: " + line);
                        }
                        return false;
                    }
                }
            }
        } catch (IOException e) {
            System.err.println();
            System.err.println("CANNOT READ " + fileName + " - ERROR " + e);
            return false;
        }
        return true;
    }

    private static String padR(String s, int n) {
        while (s.length() < n)
            s += " ";
        return s;
    }

    private static String unescape(String s) {
        if (s.equals("~"))
            return ""; // this means "an empty string"
        s = s.replace("\\s{}", " "); // this is one way to represent space
        s = s.replace("\\char`\\\\", "\\"); // this is a way to represent backslash
        s = s.replace("\\#", "#"); // this is a way to represent hash
        s = s.replace("\\_", "_"); // this is a way to represent underscore
        s = s.replace("\\^{}", "^"); // this is a way to represent ^
        StringBuilder sb = new StringBuilder(s);
        int i = 0;
        while (i < sb.length() && sb.charAt(i) == '~')
            sb.setCharAt(i++, ' ');
        return sb.toString();
    }
}
