import java.io.*;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.Stream;

/**
 * NEERC'2016 Test generator for problem A: Abbreviation
 *
 * @author Roman Elizarov
 */
@SuppressWarnings("Duplicates")
public class Tests {
    private static final String TEST_NAME_FORMAT = "%02d";
    private static final File TESTS_DIR = new File("tests");
    private static final File LST_FILE = new File(TESTS_DIR, "tests.lst");

    private static final int MAX_LINES = 1000;
    private static final int MAX_CHARS = 120;

    public static void main(String[] args) throws IOException {
        Tests tests;
        if (args.length == 0) {
            // all tests
            tests = new Tests(null);
            LST_FILE.delete();
        } else {
            // specified tests
            tests = new Tests(Stream.of(args).mapToInt(Integer::parseInt).boxed().collect(Collectors.toSet()));
        }
        TESTS_DIR.mkdir();
        tests.generate();
    }

    // -------------------------- instance impl --------------------------

    private Set<Integer> includeTests;
    private int nTests = 0;
    private final Random RND = new Random(1);

    private Tests(Set<Integer> includeTests) {
        this.includeTests = includeTests;
    }

    private void generate() throws IOException {
        write("Sample 1",
                "This is ACM North Eastern European Regional Contest,",
                "sponsored by International Business Machines.",
                "The. Best. Contest. Ever.",
                "A Great Opportunity for all contestants.");


        write("Sample 2", "ab Ab A Abc AB Abcd ABc Abcde AbC");
        write("Sample 3", "Oh  No  Extra Spaces.And,Punctuation Ruin Everything");

        write("One cap letter", "A");
        write("One small letter", "a");
        write("One small cap word", "Aa");
        write("One small all cap word", "AA");

        write("Abb pair lines",
                "Ab Cd",
                "AB Cd",
                "Ab CD",
                "Ab  Cd",
                "Ab,Cd",
                "Ab.Cd"
        );

        write("Abb triple",
                "Ab Cd Ef",
                "AB Cd Ef",
                "Ab CD Ef",
                "Ab Cd EE",
                "AB CD EE",
                "Ab  Cd Ef",
                "Ab Cd  Ef",
                "Ab  Cd  Ef",
                "Ab,Cd Ef",
                "Ab Cd,Ef",
                "Ab,Cd Ef",
                "Ab,Cd,Ef",
                "Ab.Cd Ef",
                "Ab Cd.Ef",
                "Ab.Cd.Ef"
        );

        write("One-two mix",
                "A Cd Ef",
                "Ab C Ef",
                "Ab Cd E",
                "A C Ef",
                "A Cd E",
                "Ab C E",
                "A.Cd Ef",
                "Ab.C Ef",
                "Ab.Cd E",
                "A.C Ef",
                "A.Cd E",
                "Ab.C E",
                "A.Cd.Ef",
                "Ab.C.Ef",
                "Ab.Cd.E",
                "A.C.Ef",
                "A.Cd.E",
                "Ab.C.E",
                "A Cd.Ef",
                "Ab C.Ef",
                "Ab Cd.E",
                "A C.Ef",
                "A Cd.E",
                "Ab C.E"
        );

        writeRandom(MAX_LINES, 1, 3, 0.5, 2);
        writeRandom(MAX_LINES, 1, 5, 0.5, 2);
        for (int len = 5; len <= 10; len++) {
            writeRandom(MAX_LINES, len, len, 0.5, 2);
            writeRandom(MAX_LINES, len, len, 1.0, 2);
            writeRandom(MAX_LINES, len, len, 1.0, len);
        }
        writeRandom(MAX_LINES, 1, MAX_CHARS / 2, 0.5, 2);
        writeRandom(MAX_LINES, 1, MAX_CHARS, 0.5, 2);
        writeRandom(MAX_LINES, 1, MAX_CHARS, 0.5, 2);
        writeRandom(MAX_LINES, MAX_CHARS / 2, MAX_CHARS, 0.5, 2);
        writeRandom(MAX_LINES, MAX_CHARS, MAX_CHARS, 0.5, 2);
        writeRandom(MAX_LINES, MAX_CHARS, MAX_CHARS, 1.0, 2);
        writeRandom(MAX_LINES, MAX_CHARS, MAX_CHARS, 1.0, 3);
        writeRandom(MAX_LINES, MAX_CHARS, MAX_CHARS, 1.0, 15);
        writeRandom(MAX_LINES, MAX_CHARS, MAX_CHARS, 1.0, MAX_CHARS);
        writeRandom(MAX_LINES, MAX_CHARS, MAX_CHARS, 0.9, 15);
        writeRandom(MAX_LINES, MAX_CHARS, MAX_CHARS, 0.8, 15);
    }

    private PrintWriter openTestFile(String desc) throws IOException {
        int testNo = ++nTests;
        if (includeTests == null) {
            // generate lst file when writing all tests
            try (PrintWriter lstOut = new PrintWriter(new FileWriter(LST_FILE, true))) {
                lstOut.printf(TEST_NAME_FORMAT + " %s%n", nTests, desc);
            }
        }
        if (includeTests == null || includeTests.contains(testNo)) {
            File file = new File(TESTS_DIR, String.format(TEST_NAME_FORMAT, testNo));
            PrintWriter out = new PrintWriter(file);
            System.out.printf("Writing test " + TEST_NAME_FORMAT + ": %s%n", testNo, desc);
            return out;
        } else {
            return new MockPrintWriter(); // don't write to disk
        }
    }

    private void write(String desc, String... lines) throws IOException {
        try (PrintWriter out = openTestFile(desc)) {
            for (String line : lines) {
                out.println(line);
            }
        }
    }

    private void writeRandom(int n, int minLen, int maxLen, double capWordProb, int maxWordLen) throws IOException {
        List<String> lines = new ArrayList<>();
        for (int i = 0; i < n; i++) {
            int len;
            while (true) {
                len = RND.nextInt(maxLen - minLen + 1) + minLen;
                if (len == 0 && i == n - 1) continue;
                break;
            }
            StringBuilder sb = new StringBuilder();
            while (sb.length() < len) {
                int curLen = sb.length();
                boolean space = curLen > 0;
                int extraSpace = space ? 1 : 0;
                if (curLen < len - 2 - extraSpace && RND.nextDouble() < capWordProb) {
                    if (space)
                        sb.append(' ');
                    int worldLen = Math.min(RND.nextInt(maxWordLen - 1) + 2, len - curLen - extraSpace);
                    sb.append((char)('A' + RND.nextInt(26)));
                    for (int j = 0; j < worldLen - 1; j++) {
                        sb.append((char)('a' + RND.nextInt(26)));
                    }
                } else {
                    chars: while (true) {
                        switch (RND.nextInt(4)) {
                            case 0: sb.append((char)('A' + RND.nextInt(26))); break chars;
                            case 1: sb.append((char)('a' + RND.nextInt(26))); break chars;
                            case 2: if (curLen == 0 || curLen == len - 1) continue chars; else { sb.append(' '); break chars; }
                            case 3: sb.append(RND.nextBoolean() ? ',' : '.'); break chars;
                        }
                    }
                }
            }
            lines.add(sb.toString());
        }
        write(String.format(Locale.US, "%d lines, [%d,%d] capWordProb=%.2f, maxWordLen=%d",
                n, minLen, maxLen, capWordProb, maxWordLen),
                lines.toArray(new String[0]));
    }

    private static class MockPrintWriter extends PrintWriter {
        MockPrintWriter() { super(new StringWriter()); }
    }
}
