#ifdef DBG1
  #define _GLIBCXX_DEBUG
#endif

#include <cstdio>
#include <cassert>
#include <cstring>
#include <string>
#include <vector>
#include <algorithm>
#include <map>
#include <set>

#ifdef DBG1
    #define eprintf(...) fprintf(stderr, __VA_ARGS__)
#else
    #define eprintf(...)
#endif

using namespace std;

typedef long long ll;
typedef unsigned long long ull;
typedef pair <int, int> pii;

#define TASKNAME "binary"

struct Node {
  int par;
  int child[2];
  vector <int> ends;

  Node() {
    par = -1;
    child[0] = child[1] = -1;
  }
};

template <class T>
struct ChangesVector {
private:
  T defValue;
  vector <T> v;
  vector <int> changes;
public:
  ChangesVector() {}
  ChangesVector(int n, T defValue) : defValue(defValue), v(n, defValue) {}

  T operator [] (int index) const {
    return v[index];
  }

  void set(int index, T value) {
    //assert(v[index] == defValue);
    v[index] = value;
    changes.push_back(index);
  }

  void clearChanges() {
    changes.clear();
  }

  void rollBackChanges() {
    while (!changes.empty()) {
      v[changes.back()] = defValue;
      changes.pop_back();
    }
  }
};

int addToTrie(vector <Node> & nodes,
              char *s, int num) {
  eprintf("%s %s\n", __func__, s);
  int x = 0;
  for (int i = 0; s[i]; ++i) {
    if (nodes[x].child[s[i] - '0'] == -1) {
      int newNum = int(nodes.size());
      Node newNode;
      newNode.par = x;
      nodes.push_back(newNode);
      nodes[x].child[s[i] - '0'] = newNum;
    }
    x = nodes[x].child[s[i] - '0'];
  }
  nodes[x].ends.push_back(num);
  return x;
}

vector <Node> nodes;
vector <int> num;
ChangesVector <bool> wentUp;
ChangesVector <bool> wentDown;
ChangesVector <int> ans;

bool trySetValue(int s, int value);

bool processNodeEnds(int x, int ignoreEnd) {
  eprintf("%s node %d ignoreEnd %d\n", __func__, x, ignoreEnd);
  for (int z : nodes[x].ends) {
    if (z == ignoreEnd) { continue; }
    if (!trySetValue(z / 2, (z & 1) ^ 1)) {
      return false;
    }
  }
  return true;
}

bool goUp(int x) {
  while (!wentUp[x]) {
    wentUp.set(x, true);
    x = nodes[x].par;
    if (x == -1) { break; }
    if (!processNodeEnds(x, -1)) {
      return false;
    }
  }
  return true;
}

bool goDown(int x) {
  if (wentDown[x]) { return true; }

  wentDown.set(x, true);
  for (int i = 0; i < 2; ++i) {
    int y = nodes[x].child[i];
    if (y == -1) { continue; }
    if (!processNodeEnds(y, -1)) {
      return false;
    }
    if (!goDown(y)) {
      return false;
    }
  }
  return true;
}

bool trySetValue(int s, int value) {
  eprintf("%s s %d value %d\n", __func__, s, value);
  int x = 2 * s + value;
  if (num[x] == -1) { return false; }
  if (ans[s] != -1) {
    return ans[s] == value;
  }

  ans.set(s, value);
  if (!processNodeEnds(num[x], x)) {
    return false;
  }
  if (!goUp(num[x])) {
    return false;
  }
  if (!goDown(num[x])) {
    return false;
  }
  return true;
}

void dumpTrie(const vector <Node> & nodes) {
#ifdef DBG1
  for (int i = 0; i < (int)nodes.size(); ++i) {
    eprintf("%d : par %d child0 %d child1 %d ends [", i, nodes[i].par, nodes[i].child[0], nodes[i].child[1]);
    for (int z : nodes[i].ends) {
      eprintf("%d ", z);
    }
    eprintf("]\n");
  }
#endif
}
    
int main() {
  freopen(TASKNAME ".in", "r", stdin);
  freopen(TASKNAME ".out", "w", stdout);

  static char text[2 * int(5e5)];
  char *text_ptr = text;

  int n;
  scanf("%d", &n);

  vector <char*> s(n);
  vector <int> pos(n);
  for (int i = 0; i < n; ++i) {
    s[i] = text_ptr;
    scanf("%s", s[i]);
    int len = strlen(s[i]);
    text_ptr = s[i] + len + 1;

    char *curPos = strchr(s[i], '?');
    pos[i] = (curPos ? curPos - s[i] : -1);
  }

  nodes.push_back(Node());
  num.assign(2 * n, -1);
  for (int i = 0; i < n; ++i) {
    if (pos[i] != -1) {
      for (int j = 0; j < 2; ++j) {
        s[i][pos[i]] = '0' + j;
        num[2 * i + j] = addToTrie(nodes, s[i], 2 * i + j);
      }
      s[i][pos[i]] = '?';
    } else {
      num[2 * i] = addToTrie(nodes, s[i], 2 * i);
    }
  }

  //dumpTrie(nodes);

  ans = ChangesVector<int>(n, -1);
  wentUp = ChangesVector<bool>(nodes.size(), false);
  wentDown = ChangesVector<bool>(nodes.size(), false);

  vector <int> q;
  q.push_back(0);
  for (int i = 0; i < int(q.size()); ++i) {
    int x = q[i];
    for (int j = 0; j < 2; ++j) {
      int y = nodes[x].child[j];
      if (y == -1) { continue; }
      wentUp.set(y, true);
      if (nodes[y].ends.empty()) {
        q.push_back(y);
      }
    }
  }

  for (int i = 0; i < n; ++i) {
    ans.clearChanges();
    wentUp.clearChanges();
    wentDown.clearChanges();
    if (!trySetValue(i, 0)) {
      eprintf("rollBackChanges\n");
      ans.rollBackChanges();
      wentUp.rollBackChanges();
      wentDown.rollBackChanges();
      if (!trySetValue(i, 1)) {
        printf("NO\n");
        return 0;
      }
    }
  }

  printf("YES\n");
  for (int i = 0; i < n; ++i) {
    if (pos[i] != -1) {
      assert(0 <= ans[i] && ans[i] <= 1);
      s[i][pos[i]] = '0' + ans[i];
    } else {
      assert(ans[i] == 0);
    }
    printf("%s\n", s[i]);
  }

  return 0;
}
