
import java.io.*
import kotlin.system.measureTimeMillis

/**
 * Solution for NEERC'2016 Problem B: Binary Code
 * This solution requires correctness of the input.
 * @author Roman Elizarov
 */

const val INPUT = "binary.in"
const val OUTPUT = "binary.out"

const val MAX_N = 500000
const val MAX_TOTAL = 500000

const val DEBUG_GRAPH = false

fun main(args: Array<String>) {
    val time = measureTimeMillis {
        val input = Scanner(File(INPUT)).use { it.readInput() }
        val output = input.solve()
        PrintWriter(OUTPUT).use { writeOutput(it, output) }
    }
    println("Done in $time ms")
}

// input

class Input(val w: List<String>)

fun Scanner.readInput() : Input {
    val res = mutableListOf<String>()
    val time = measureTimeMillis {
        val n = nextInt()
        nextLine()
        require(n in 1..MAX_N)
        var total = 0
        for (i in 0..n - 1) {
            val s = next()
            nextLine()
            require(!s.isEmpty())
            var qc = 0
            for (c in s) when (c) {
                '0', '1' -> {}
                '?' -> qc++
                else -> throw IllegalArgumentException("Char $c")
            }
            require(qc <= 1)
            total += s.length
            require(total <= MAX_TOTAL)
            res.add(s)
        }
    }
    println("Read input in $time ms")
    return Input(res)
}

// solution

class Result(val r: List<String>?)

val NO = Result(null)

const val T = 0
const val F = 1

class Trie {
    var hasTrue: Boolean = false
    var terminals: MutableList<Int>? = null
    var excludes: List<Int>? = null
    var left: Trie? = null
    var right: Trie? = null
}

class SRef(val t: Trie, var step: Int = 0)

fun Input.solve(): Result {
    val imps = Graph()

    check(imps.addV() == T)
    check(imps.addV() == F)

    val n = w.size
    val v0 = IntArray(n)
    val v1 = IntArray(n)
    for (i in 0..n - 1) {
        v0[i] = imps.addV()
        v1[i] = imps.addV()
    }
    // build tries & init empty implications graph
    val root = Trie()
    val trieTime = measureTimeMillis {
        for ((index, s) in w.withIndex()) {
            if (!root.addToTrie(s, 0, v0[index], '?')) return NO
        }
    }
    println("Build trie in $trieTime ms")

    // virtual nodes factory
    fun vr(): Int {
        val a = imps.addV()
        imps.addV()
        return a
    }

    // scan and add implication edges
    val implTime = measureTimeMillis {
        val stack = mutableListOf(SRef(root, 0))
        while (!stack.isEmpty()) {
            val sp = stack.last()
            when (sp.step++) {
                0 -> sp.t.left?.let { stack.add(SRef(it))  }
                1 -> sp.t.right?.let { stack.add(SRef(it)) }
                2 -> {
                    val cs = arrayListOf<Int>()
                    sp.t.left?.let { it.excludes?.let { cs.addAll(it) } }
                    sp.t.right?.let { it.excludes?.let { cs.addAll(it) } }
                    stack.removeAt(stack.lastIndex)
                    val vs = sp.t.terminals
                    if (vs != null) {
                        val k = vs.size
                        var exclude: Int
                        if (k == 1) {
                            exclude = vs[0]
                        } else {
                            val r = Array(k) { vr() }
                            for (j in 0..k - 1) {
                                imps.then(vs[j], r[j])
                                if (j < k - 1) {
                                    imps.nand(vs[j], r[j + 1])
                                    imps.then(r[j + 1], r[j])
                                }
                            }
                            exclude = r[0]
                        }
                        for (b in cs) {
                            imps.nand(exclude, b)
                        }
                        sp.t.excludes = listOf(exclude)
                    } else {
                        sp.t.excludes = cs
                    }
                }
            }
        }
        // add F -> T implication
        imps.implies(F, T)
    }
    println("Build implication graph in $implTime ms with ${imps.size} vertices and ${imps.edgesSize} edges")

    // strongly connected components with Kosaraju & Sharir algorithm
    val comp = IntArray(imps.size)
    val allT = mutableSetOf<Int>()

    if (DEBUG_GRAPH) println("---")
    var reversedGraph = Graph()
    val revTime = measureTimeMillis {
        reversedGraph = reverseGraph(imps)
    }
    println("Reversed graph in $revTime ms")

    val compTime = measureTimeMillis {
        var curComp = 0
        val dfs = Dfs(imps)
        val initialOrder = IntArray(imps.size) { i -> i }
        val outOrder = dfs.invoke(initialOrder) { outOrder, fromIndex, toIndex ->
            if (outOrder[toIndex - 1] == T)
                for (i in fromIndex..toIndex - 1)
                    allT.add(outOrder[i])
        }
        val reversedOrder = IntArray(imps.size) { i -> outOrder[outOrder.size - i - 1] }
        dfs.reuse(reversedGraph)
        dfs.invoke(reversedOrder) { outOrder, fromIndex, toIndex ->
            val cc = ++curComp
            for (i in fromIndex..toIndex - 1)
                comp[outOrder[i]] = cc
        }
    }
    println("Build connected components in $compTime ms")

    // check true/false contradictions
    if (allT.contains(F)) return NO
    // resolve 2SAT vars
    val r = mutableListOf<String>()
    for (i in 0..n - 1) {
        val s = w[i]
        val qi = s.indexOf('?')
        if (qi < 0) {
            r.add(s)
            continue
        }
        val c0 = comp[v0[i]] // char '0'
        val c1 = comp[v1[i]] // char '1'
        if (c0 == c1) return NO
        var v : Int
        if (c0 > c1) {
            v = 0
        } else {
            v = 1
        }
        r.add(s.substring(0, qi) + v + s.substring(qi + 1))
    }
    return Result(r)
}

fun reverseGraph(imps: Graph): Graph {
    val rev = Graph()
    for (v in 0..imps.size - 1) {
        rev.addV()
    }
    for (v in 0..imps.size - 1) {
        var k = imps.first(v)
        while (k >= 0) {
            val u = imps.vertex(k)
            rev.implies(u, v)
            k = imps.next(k)
        }
    }
    return rev
}

private fun Graph.nand(a: Int, b: Int) {
    if (DEBUG_GRAPH) println("$a nand $b")
    implies(a, b xor 1)
    implies(b, a xor 1)
}

private fun Graph.then(a: Int, b: Int) {
    if (DEBUG_GRAPH) println("$a then $b")
    implies(a, b)
    implies(b xor 1, a xor 1)
}

class Graph {
    var size = 0
    var edgesSize = 0
    var fl: IntArray = IntArray(10) // first/last
    var vn: IntArray = IntArray(10) // vertex/n

    fun first(v: Int) = fl[2 * v]
    fun last(v: Int) = fl[2 * v + 1]

    fun vertex(k: Int) = vn[2 * k]
    fun next(k: Int) = vn[2 * k + 1]

    fun addV(): Int {
        val i = size++
        if (size * 2 > fl.size) fl = fl.copyOf(fl.size * 2)
        fl[2 * i] = -1
        fl[2 * i + 1] = -1
        return i
    }

    fun implies(a: Int, b: Int) {
        if (DEBUG_GRAPH) println("$a -> $b")
        val j = edgesSize++
        if (edgesSize * 2 > vn.size) vn = vn.copyOf(vn.size * 2)
        vn[2 * j] = b
        vn[2 * j + 1] = -1
        val k = last(a)
        if (k < 0) {
            fl[2 * a] = j
        } else {
            vn[2 * k + 1] = j
        }
        fl[2 * a + 1] = j
    }
}

class Dfs(var imps: Graph) {
    val visited = BooleanArray(imps.size)
    val svc = IntArray(imps.size * 2) // stack vertex cur
    var sp = -1
    val outOrder = IntArray(imps.size)

    fun reuse(newImps: Graph) {
        require(sp == -1)
        visited.fill(false)
        imps = newImps
    }

    fun add(v: Int) {
        visited[v] = true
        sp++
        svc[2 * sp] = v
        svc[2 * sp + 1] = imps.first(v)
    }

    fun invoke(order: IntArray, block: (IntArray, Int, Int) -> Unit): IntArray {
        var outIndex = 0
        for (a in order) {
            if (visited[a]) continue
            val startIndex = outIndex
            add(a)
            while (sp >= 0) {
                val k = svc[2 * sp + 1]
                if (k >= 0) {
                    val v = imps.vertex(k)
                    svc[2 * sp + 1] = imps.next(k)
                    if (!visited[v])
                        add(v)
                } else {
                    outOrder[outIndex++] = svc[2 * sp]
                    sp--
                }
            }
            block(outOrder, startIndex, outIndex)
        }
        return outOrder
    }
}

private fun Trie.addToTrie(s: String, from: Int, v0: Int, ch: Char): Boolean {
    var t = this
    for (i in from..s.length - 1) when (s[i]) {
        '0' -> t = left(t)
        '1' -> t = right(t)
        '?' -> {
            left(t).addToTrie(s, i + 1, v0, '0')
            right(t).addToTrie(s, i + 1, v0, '1')
            return true
        }
    }
    when (ch) {
        '0' -> terminals(t).add(v0)
        '1' -> terminals(t).add(v0 + 1)
        '?' -> {
            if (t.hasTrue) return false
            t.hasTrue = true
            terminals(t).add(T)
        }
        else -> throw IllegalStateException()
    }
    return true
}

private fun terminals(t: Trie) = t.terminals ?: arrayListOf<Int>().apply { t.terminals = this }

private fun right(t: Trie) = t.right ?: Trie().apply { t.right = this }
private fun left(t: Trie) = t.left ?: Trie().apply { t.left = this }

// result

fun writeOutput(out: PrintWriter, result: Result) {
    val time = measureTimeMillis {
        if (result.r == null) {
            out.println("NO")
        } else {
            out.println("YES")
            result.r.forEach { out.println(it) }
        }
    }
    println("Write output in $time ms")
}

//----------------- utility ------------------

fun fail(s: String): Nothing = throw IllegalArgumentException(s)

//----------------- just for validation ------------------

/**
 * Strict scanner to verify 100% correspondence between input files and input file format specification.
 * It is a drop-in replacement for [java.util.Scanner] that could be added to a solution source
 * (cut-and-paste) without breaking its ability to work with [java.util.Scanner].
 */
class Scanner(source: File) : Closeable {
    private val reader = BufferedReader(FileReader(source))
    private var line: String? = ""
    private var pos: Int = 0
    private var lineNo: Int = 0

    init { nextLine() }

    override fun close() {
        require(line == null) { "Extra data at the end of file" }
        try {
            reader.close()
        } catch (e: IOException) {
            fail("Failed to close with " + e)
        }

    }

    fun nextLine() {
        require(line != null) { "EOF" }
        require(pos == line!!.length) { "Extra characters on line " + lineNo }
        try {
            line = reader.readLine()
        } catch (e: IOException) {
            fail("Failed to read line with " + e)
        }
        pos = 0
        lineNo++
    }

    operator fun next(): String {
        require(line != null) { "EOF" }
        require(line!!.isNotEmpty()) { "Empty line " + lineNo }
        if (pos == 0)
            require(line!![0] > ' ') { "Line $lineNo starts with whitespace" }
        else {
            require(pos < line!!.length) { "Line $lineNo is over" }
            require(line!![pos] == ' ') { "Wrong whitespace on line " + lineNo }
            pos++
            require(pos < line!!.length) { "Line $lineNo is over" }
            require(line!![pos] > ' ') { "Line $lineNo has double whitespace" }
        }
        val sb = StringBuilder()
        while (pos < line!!.length && line!![pos] > ' ')
            sb.append(line!![pos++])
        return sb.toString()
    }

    fun nextInt(): Int {
        val s = next()
        require(s.length == 1 || s[0] != '0') { "Extra leading zero in number $s on line $lineNo" }
        require(s[0] != '+') { "Extra leading '+' in number $s on line $lineNo" }
        try {
            return Integer.parseInt(s)
        } catch (e: NumberFormatException) {
            fail("Malformed number $s on line $lineNo")
        }

    }
}
