import ru.ifmo.testlib.Checker;
import ru.ifmo.testlib.InStream;
import ru.ifmo.testlib.Outcome;

import java.util.*;

import static ru.ifmo.testlib.Outcome.Type.*;

/**
 * Checker for NEERC'2016 Problem C: Cactus Construction.
 *
 * @author Pavel Kunyavskiy
 */
public class Check implements Checker {

    private final static int MAX_OPS = 1_000_000;

    private class Edge {
        int u, v;

        Edge(int u, int v) {
            if (u < v) {
                this.u = u;
                this.v = v;
            } else {
                this.v = u;
                this.u = v;
            }
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            Edge edge = (Edge) o;

            return u == edge.u && v == edge.v;

        }

        @Override
        public int hashCode() {
            int result = u;
            result = 31 * result + v;
            return result;
        }
    }

    private HashSet<Edge> needEdges;
    private HashSet<Edge> builtEdges;

    private GraphPart[] parts;

    private class GraphPart {
        List<Integer>[] vertices;

        GraphPart(int v) {
            vertices = new List[4];
            for (int i = 0; i < 4; i++) {
                vertices[i] = new ArrayList<>();
            }
            vertices[0].add(v);
        }

        int size() {
            int tot = 0;
            for (int i = 0; i < 4; i++) {
                tot += vertices[i].size();
            }
            return tot;
        }

        void recolor(int c1, int c2) {
            if (vertices[c1].size() > vertices[c2].size()) {
                List<Integer> t = vertices[c1];
                vertices[c1] = vertices[c2];
                vertices[c2] = t;
            }
            vertices[c2].addAll(vertices[c1]);
            vertices[c1].clear();
        }
    }



    private int readInput(InStream inf) {
        int n = inf.nextInt();
        int m = inf.nextInt();
        needEdges = new HashSet<>();
        builtEdges = new HashSet<>();
        for (int i = 0; i < m; i++) {
            int k = inf.nextInt();
            int last = inf.nextInt() - 1;
            for (int j = 1; j < k; j++) {
                int next = inf.nextInt() - 1;
                needEdges.add(new Edge(last, next));
                last = next;
            }
        }
        return n;
    }

    private static final int JOIN_OP = 0;
    private static final int RECOLOR_OP = 1;
    private static final int CONNECT_OP = 2;

    private class Operation {
        int type;
        int arg1, arg2, arg3;

        Operation(int type, int arg1, int arg2, int arg3) {
            this.type = type;
            this.arg1 = arg1;
            this.arg2 = arg2;
            this.arg3 = arg3;
        }
    }

    private Operation[] readOutput(InStream ouf) {
        int ops = ouf.nextInt();
        if (ops < 0 || ops > MAX_OPS) {
            throw new Outcome(WA, "Bad number of operations" + ops);
        }
        Operation[] res = new Operation[ops];
        for (int i = 0; i < ops; i++) {
            String name = ouf.nextToken();
            switch (name) {
                case "j": {
                    int u = readVertex(ouf);
                    int v = readVertex(ouf);
                    res[i] = new Operation(JOIN_OP, u, v, 0);
                    break;
                }
                case "r": {
                    int v = readVertex(ouf);
                    int c1 = readColor(ouf);
                    int c2 = readColor(ouf);
                    res[i] = new Operation(RECOLOR_OP, v, c1, c2);
                    break;
                }
                case "c": {
                    int v = readVertex(ouf);
                    int c1 = readColor(ouf);
                    int c2 = readColor(ouf);
                    res[i] = new Operation(CONNECT_OP, v, c1, c2);
                    break;
                }
                default:
                    throw new Outcome(WA, "Unknown operation " + name);
            }
        }
        return res;
    }

    private int readVertex(InStream ouf) {
        int v = ouf.nextInt();
        if (v < 1 || v > parts.length) {
            throw new Outcome(WA, "Bad vertex id " + v);
        }
        return v - 1;
    }

    private int readColor(InStream ouf) {
        int c = ouf.nextInt();
        if (c < 1 || c > 4) {
            throw new Outcome(WA, "Bad color id " + c);
        }
        return c - 1;
    }


    public Outcome test(InStream inf, InStream ouf, InStream ans) {
        int n = readInput(inf);

        parts = new GraphPart[n];
        for (int i = 0; i < n; i++) {
            parts[i] = new GraphPart(i);
        }

        Operation[] ops = readOutput(ouf);

        for (int i = 0; i < ops.length; i++) {
            Operation op = ops[i];
            try {
                switch (op.type) {
                    case JOIN_OP:
                        processJoin(op.arg1, op.arg2);
                        break;
                    case RECOLOR_OP:
                        processRecolor(op.arg1, op.arg2, op.arg3);
                        break;
                    case CONNECT_OP:
                        processConnect(op.arg1, op.arg2, op.arg3);
                        break;
                }
            } catch (Outcome e) {
                return new Outcome(e.getType(), e.getComment() + " (operation " + (i + 1) + ")");
            }
        }

        for (Edge e : needEdges) {
            if (!builtEdges.contains(e)) {
                return new Outcome(WA, "Edge (" + (e.v + 1) + " " + (e.u + 1) + ") not built at the end");
            }
        }
        return new Outcome(OK, "Cactus is constructed in " + ops.length + " operations");
    }

    private void processConnect(int v, int c1, int c2) {
        for (int v1 : parts[v].vertices[c1]) {
            for (int v2 : parts[v].vertices[c2]) {
                if (c1 != c2 || v1 < v2) {
                    buildEdge(v1, v2);
                }
            }
        }
    }

    private void buildEdge(int v1, int v2) {
        Edge edge = new Edge(v1, v2);
        if (!needEdges.contains(edge)) {
            throw new Outcome(WA, "Edge " + (v1 + 1) + " " + (v2 + 1) + " is not in graph, but was built");
        }
        if (!builtEdges.add(edge)) {
            throw new Outcome(WA, "Edge " + (v1 + 1) + " " + (v2 + 1) + " was built twice");
        }
    }

    private void processRecolor(int v, int c1, int c2) {
        if (c1 == c2) return;
        parts[v].recolor(c1, c2);
    }

    private void processJoin(int u, int v) {
        if (parts[u] == parts[v]) {
            throw new Outcome(WA, "Join for vertices in same component");
        }
        if (parts[u].size() > parts[v].size()) {
            int t = u; u = v; v = t;
        }
        GraphPart part = parts[u];
        for (int i = 0; i < 4; i++) {
            for (int vs : part.vertices[i]) {
                parts[vs] = parts[v];
                parts[v].vertices[i].add(vs);
            }
        }
    }
}
