import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.Random;

public class Tests {

    private static final int MAX_N = 100000;
    private static final int MAX_M = 200000;

    public static void main(String[] args) {
        new Tests().gen();
    }

    private int testId = 1;

    private Random rnd;
    private PrintWriter comments;

    private void writeTest(TestCase test) {
        try {
            String comment = "Test " + testId + ": " + test.description;
            comments.println(comment);
            System.out.println(comment);
            PrintWriter out = new PrintWriter(String.format("tests/%02d", testId++));
            test.print(out);
            out.close();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    private void myShuffle(int[] p) {
        for (int i = 0; i < p.length; i++) {
            int x = rnd.nextInt(i + 1);
            int t = p[x];
            p[x] = p[i];
            p[i] = t;
        }
    }

    private class Arc {
        int from, to;

        Arc(int from, int to) {
            this.from = from;
            this.to = to;
        }

        @Override
        public boolean equals(Object o) {
            if (this == o) return true;
            if (o == null || getClass() != o.getClass()) return false;

            Arc arc = (Arc) o;

            return from == arc.from && to == arc.to;

        }

        @Override
        public int hashCode() {
            int result = from;
            result = 31 * result + to;
            return result;
        }
    }

    private class TestCase {
        int n;
        ArrayList<Arc> arcs;
        HashSet<Arc> set;
        boolean noShuffle;
        String description;

        TestCase(int n) {
            this.n = n;
            arcs = new ArrayList<>();
            set = new HashSet<>();
            noShuffle = false;
        }

        void print(PrintWriter out) {
            if (!noShuffle) {
                int[] p = new int[n];
                for (int i = 0; i < n; i++) {
                    p[i] = i;
                }
                myShuffle(p);
                for (Arc arc : arcs) {
                    arc.from = p[arc.from];
                    arc.to = p[arc.to];
                }
                Collections.shuffle(arcs, rnd);
            }

            out.println(n + " " + arcs.size());
            for (Arc arc : arcs) {
                out.println((arc.from + 1) + " " + (arc.to + 1));
            }
        }

        boolean addArc(int i, int j) {
            Arc arc = new Arc(i, j);
            return addArc(arc);
        }

        boolean addArc(Arc arc) {
            if (set.contains(arc)) {
                return false;
            }
            set.add(arc);
            arcs.add(arc);
            return true;
        }
    }

    private TestCase genSample() {
        TestCase res = new TestCase(6);
        res.description = "Sample test";
        res.noShuffle = true;
        res.addArc(0, 1);
        res.addArc(1, 0);
        res.addArc(1, 2);
        res.addArc(0, 3);
        res.addArc(3, 0);
        res.addArc(3, 4);
        res.addArc(4, 5);
        return res;
    }

    private TestCase[] genAll(int n, int maxN, int maxM) {
        ArrayList<TestCase> ts = new ArrayList<>();
        TestCase t = new TestCase(0);
        for (int i = 0; i < (1 << (n * n)); i++) {
            if (i == 0 && n == maxN) continue;
            if (t.n + n > maxN || t.arcs.size() + n * n > maxM) {
                ts.add(t);
                t = new TestCase(0);
            }
            t.n += n;
            for (int j = 0; j < n; j++) {
                for (int k = 0; k < n; k++) {
                    if ((i & (1 << (j * n + k))) != 0) {
                        t.addArc(t.n + j - n, t.n + k - n);
                    }
                }
            }
        }
        ts.add(t);
        TestCase[] ret = new TestCase[ts.size()];
        ts.toArray(ret);
        return ret;
    }

    private TestCase genRandom(int maxn, int maxm, int maxtotn, int maxtotm) {
        TestCase t = new TestCase(0);
        t.description = "Random test with parts <= " + maxn + " vertices, " + maxm + " edges";
        while (maxtotn > 1 && maxtotm > 1) {
            maxm = Math.min(maxm, maxtotm);
            maxn = Math.min(maxn, maxtotn);
            int n = rnd.nextInt(maxn - maxn / 2) + maxn / 2;
            int m = rnd.nextInt(Math.min(maxm, n * n));
            maxtotn -= n;
            maxtotm -= m;
            addRandomSubgraph(t, n, m);
        }
        return t;
    }

    private void addRandomSubgraph(Tests.TestCase t, int n, int m) {
        int off = t.n;
        t.n += n;
        assert n * n >= m;
        if ((long) n * n >= 2 * MAX_M) {
            for (int i = 0; i < m; i++) {
                int u = rnd.nextInt(n);
                int v = rnd.nextInt(n);
                if (!t.addArc(u + off, v + off)) {
                    i--;
                }
            }
        } else {
            ArrayList<Arc> all = new ArrayList<>();
            for (int i = 0; i < n; i++) {
                for (int j = 0; j < n; j++) {
                    all.add(new Tests.Arc(i, j));
                }
            }
            Collections.shuffle(all, rnd);
            for (int i = 0; i < m; i++) {
                t.addArc(all.get(i).from + off, all.get(i).to + off);
            }
        }
    }

    private TestCase genRandom(int n, int m) {
        TestCase t = new TestCase(0);
        addRandomSubgraph(t, n, m);
        t.description = "Random test with " + n + " vertices, " + m + " edges";
        return t;
    }

    private TestCase genBamboo(int n) {
        TestCase t = new TestCase(n);
        for (int i = 0; i < n - 1; i++) {
            t.addArc(i, i + 1);
        }
        t.description = "Bamboo with " + n + " vertices";
        return t;
    }

    private TestCase genDirectedGraph(int n, int m) {
        TestCase t = new TestCase(n);
        for (int i = 0; i < n - 1; i++) {
            int u = rnd.nextInt(t.n);
            int v = rnd.nextInt(t.n);
            if (u < v) {
                int tmp = u;
                u = v;
                v = tmp;
            }
            if (!t.addArc(v, u)) {
                i--;
            }
        }
        t.description = "Directed graph with " + n + " vertices and " + m + " edges";
        return t;
    }

    private TestCase addRandomUpEdges(TestCase t, int m) {
        for (int i = 0; i < m; i++) {
            int u = rnd.nextInt(t.n);
            int v = rnd.nextInt(t.n);
            if (u < v) {
                int tmp = u;
                u = v;
                v = tmp;
            }
            if (!t.addArc(u, v)) {
                i--;
            }
        }
        t.description += " and " + m + " edges up";
        return t;
    }

    private TestCase addRandomLoops(TestCase t, int m, boolean last) {
        if (m > 0) {
            int[] x = new int[t.n];
            for (int i = 0; i < t.n; i++) {
                x[i] = i;
            }
            myShuffle(x);
            for (int i = 0; i < m; i++) {
                t.addArc(i, i);
            }
            t.description += " and " + m + " random loops";
        }
        if (last) {
            t.addArc(t.n - 1, t.n - 1);
            t.description += " and loop at the end";
        }
        return t;
    }

    private TestCase addAllLoops(TestCase t, int m) {
        for (int i = 0; i < m; i++) {
            t.addArc(i, i);
        }
        t.description += " and " + m + " loops";
        return t;
    }


    private TestCase spectialTest(int n, int m, int mask) {
        TestCase t = new TestCase(1);
        if ((mask & 4) != 0) t.addArc(0, 0);
        while (t.arcs.size() + 1 < m && t.n < n) {
            if (t.n > 1) {
                t.addArc(t.n, t.n - 1);
            }
            if (t.arcs.size() == m) {
                t.n++;
                continue;
            }

            if ((mask & (1 << (t.n & 1))) != 0) {
                t.addArc(0, t.n);
            }
            t.n++;
        }
        if ((mask & 8) != 0) {
          t.addArc(2, t.n % 2 == 0 ? (t.n - 2) : (t.n - 1));
        }
        t.noShuffle = true;
        if ((mask & 16) != 0) {
          Collections.reverse(t.arcs);
        }
        t.description = "Test against pkun_tl3-like solutions. mask = " + mask;
        return t;
    }

    private TestCase genConnectedTriangles(int n) {
        TestCase t = new TestCase(n);
        for (int i = 0; i < n - 1; i++) {
            t.addArc(i, i + 1);
        }
        for (int i = 0; i + 2 < n; i += 3) {
            t.addArc(i + 2, i);
        }
        t.description = (n / 3) + " triangles connected in line";
        return t;
    }

    private void gen() {
        new File("tests").mkdir();
        try {
            comments = new PrintWriter(new File("tests/tests.lst"));
        } catch (FileNotFoundException e) {
            e.printStackTrace();
        }
        rnd = new Random(239);
        writeTest(genSample());
        {
            TestCase t = new TestCase(1);
            t.addArc(0, 0);
            t.description = "Min test. Single loop.";
            writeTest(t);
        }
        {
            TestCase[] ts = genAll(2, 2, MAX_M);
            for (int i = 0; i < ts.length; i++) {
                TestCase t = ts[i];
                t.description = "All tests with n = 2. Part " + (i + 1);
                writeTest(t);
            }
        }
        {
            TestCase[] ts = genAll(4, MAX_N, MAX_M);
            for (int i = 0; i < ts.length; i++) {
                TestCase t = ts[i];
                t.description = "All tests with n = 4. Part " + (i + 1);
                writeTest(t);
            }
        }

        {
            int[] ns = {5 , 10, 10 , 100, 100 , 1000, 1000};
            int[] ms = {25, 30, 100, 500, 1000, 2000, 10000};
            for (int i = 0; i < ns.length; i++) {
                writeTest(genRandom(ns[i], ms[i], MAX_N, MAX_M));
            }
        }
        writeTest(genRandom(MAX_N, MAX_M));
        writeTest(genBamboo(MAX_N));
        writeTest(addRandomUpEdges(genBamboo(MAX_N), 100));
        writeTest(addRandomUpEdges(genBamboo(MAX_N), 1000));
        writeTest(addRandomUpEdges(genBamboo(MAX_N), 10000));
        writeTest(addRandomLoops(genBamboo(MAX_N), 0, true));
        writeTest(addRandomLoops(genBamboo(MAX_N), 100, false));
        writeTest(addRandomLoops(genBamboo(MAX_N), 1000, false));
        writeTest(addRandomLoops(genBamboo(MAX_N), 10000, false));
        writeTest(addRandomUpEdges(addRandomLoops(genBamboo(MAX_N), 1000, false), 1000));
        writeTest(genDirectedGraph(MAX_N, MAX_M));
        writeTest(genDirectedGraph(MAX_N / 3, MAX_M));
        writeTest(addRandomLoops(genDirectedGraph(MAX_N, MAX_M - 10000), 100, false));
        writeTest(addRandomLoops(genDirectedGraph(MAX_N / 3, MAX_M - 10000), 100, false));
        writeTest(addRandomUpEdges(genDirectedGraph(MAX_N, MAX_M - 10000), 1000));
        writeTest(addRandomUpEdges(genDirectedGraph(MAX_N / 3, MAX_M - 10000), 1000));

        for (int i = 0; i < 32; i++) {
          writeTest(spectialTest(MAX_N, MAX_M, i));
        }
        
        writeTest(addAllLoops(genBamboo(MAX_N), MAX_N - 1));
        writeTest(genConnectedTriangles(MAX_N));
        comments.close();
    }

}
