import java.io.*;
import java.util.*;
import java.util.stream.Collectors;
import java.util.stream.IntStream;
import java.util.stream.Stream;

/**
 * NEERC'2016 Test generator for problem H: Hard Refactoring
 *
 * @author Roman Elizarov
 */
public class Tests {
    private static final String TEST_NAME_FORMAT = "%02d";
    private static final File TESTS_DIR = new File("tests");
    private static final File LST_FILE = new File(TESTS_DIR, "tests.lst");

    private static final Integer MAX = (1 << 15) - 1;
    private static final Integer MIN = -(1 << 15);
    private static final Integer N = 1000;

    public static void main(String[] args) throws IOException {
        Tests tests;
        if (args.length == 0) {
            // all tests
            tests = new Tests(null);
            LST_FILE.delete();
        } else {
            // specified tests
            tests = new Tests(Stream.of(args).mapToInt(Integer::parseInt).boxed().collect(Collectors.toSet()));
        }
        TESTS_DIR.mkdir();
        tests.generate();
    }

    // -------------------------- instance impl --------------------------

    private Set<Integer> includeTests;
    private int nTests = 0;
    private final Random RND = new Random(1);

    private Tests(Set<Integer> includeTests) {
        this.includeTests = includeTests;
    }

    private void generate() throws IOException {
        write("Sample 1", range(5, 10), range(7, 20), le(2), range(21, 25), range(8, 10), ge(100));
        write("Sample 2", range(10, 0));
        write("Sample 3", le(10), ge(0));
        write("Sample 3", ge(MIN));

        // various one-clause cases
        write("One less", le(42));
        write("One more", ge(134));
        write("Less negative", le(-44));
        write("One range", range(64, 128));
        write(">= max", ge(MAX));
        write("<= min", le(MIN));
        write("<= max", le(MAX));
        write("[min,max] range", range(MIN, MAX));
        write("[max,min] range", range(MAX, MIN));
        write("[0,max] range", range(0, MAX));
        write("[0,min] range", range(0, MIN));
        write("[min,0] range", range(MIN, 0));
        write("[max,0] range", range(MAX, 0));
        write("[0,0] range", range(0, 0));
        write("[max,max] range", range(MAX, MAX));
        write("[min,min] range", range(MIN, MIN));
        write("[max-1,max] range", range(MAX-1, MAX));
        write("[max,max-1] range", range(MAX, MAX-1));
        write("[min+1,min] range", range(MIN+1, MIN));
        write("[min,min+1] range", range(MIN, MIN+1));

        // various two-clause cases
        write("[10,20]|>= 30", range(10, 20), ge(30));
        write("[10,20]|<= 30", range(10, 20), le(30));
        write(">= 15|[5,5]", ge(15), range(5, 5));
        write("<= 13|[10,15]", le(13), range(10, 15));
        write("[11,11]|[12,12]", range(11, 11), range(12, 12));
        write("[-10,0]|[0,10]", range(-10, 0), range(0, 10));
        write("[-10,10]|[-5,5]", range(-10, 10), range(-5,5));
        write("[5,10]|[5,0]", range(5, 10), range(5, 0));
        write("[18,30]|[17,20]", range(18,30), range(17, 20));

        // big repeats
        write("1000*[0,0]", IntStream.range(0, N).mapToObj(i -> range(0, 0)).toArray(String[]::new));
        write("1000*[min,max]", IntStream.range(0, N).mapToObj(i -> range(MIN, MAX)).toArray(String[]::new));
        write("1000*[max,min]", IntStream.range(0, N).mapToObj(i -> range(MAX, MIN)).toArray(String[]::new));

        // big constructive
        write("1000*[i,i]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> range(i, i)).toArray(String[]::new));
        write("1000*[i,i+1]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> range(i, i+1)).toArray(String[]::new));
        write("1000*[i,i-1]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> range(i, i-1)).toArray(String[]::new));
        write("1000*[i-1,i+1]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> range(i-1, i+1)).toArray(String[]::new));
        write("1000*[0,i]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> range(0, i)).toArray(String[]::new));
        write("1000*[i,0]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> range(i, 0)).toArray(String[]::new));
        write("1000*[>=i||<=-i]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> (i % 2 == 0) ? le(-i) : ge(i)).toArray(String[]::new));
        write("1000*[>=i||<=-i-1]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> (i % 2 == 0) ? le(-i-1) : ge(i)).toArray(String[]::new));
        write("1000*[-i,i]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> range(-i, i)).toArray(String[]::new));
        write("1000*[i,-i]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> range(i, -i)).toArray(String[]::new));
        write("1000*[2*i,2*i+1]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> range(2*i, 2*i+1)).toArray(String[]::new));
        write("1000*[3*i,3*i+1]", Mode.WITH_SHUFFLED, IntStream.range(0, N).mapToObj(i -> range(3*i, 3*i+1)).toArray(String[]::new));

        // randoms smaller range and large range
        writeRandom("Only <=", N, -1000, 1000, 1.0, 0.0, 0.0, 0.0);
        writeRandom("Only >=", N, -1000, 1000, 0.0, 1.0, 0.0, 0.0);
        writeRandom("Only ranges", N, -1000, 1000, 0.0, 0.0, 0.0, 0.0);
        writeRandom("range|<=10%", N, -1000, 1000, 0.1, 0.0, 0.0, 0.0);
        writeRandom("range|>=10%", N, -1000, 1000, 0.0, 0.1, 0.0, 0.0);
        writeRandom("Random ranges 10%", N, -1000, 1000, 0.0, 0.0, 0.1, 0.1);
        writeRandom("Random ranges borders 1%", N, -1000, 1000, 0.0, 0.0, 0.01, 0.01);

        // randoms full range
        writeRandom("Only <=", N, MIN, MAX, 1.0, 0.0, 0.0, 0.0);
        writeRandom("Only >=", N, MIN, MAX, 0.0, 1.0, 0.0, 0.0);
        writeRandom("Only ranges", N, MIN, MAX, 0.0, 0.0, 0.0, 0.0);
        writeRandom("range|<=10%", N, MIN, MAX, 0.1, 0.0, 0.0, 0.0);
        writeRandom("range|>=10%", N, MIN, MAX, 0.0, 0.1, 0.0, 0.0);
        writeRandom("Random ranges borders 10%", N, MIN, MAX, 0.0, 0.0, 0.1, 0.1);
        writeRandom("Random ranges borders 1%", N, MIN, MAX, 0.0, 0.0, 0.01, 0.01);

        // specials
        write("Random 998*{-1000,1000}|<=-2000|>=2000", Mode.SHUFFLED,
                Stream.concat(streamRandomLines(998, -1000, 1000, 0.0, 0.0, 0.0, 0.0),
                        Stream.of(le(-2000), ge(2000))).toArray(String[]::new));
    }

    private String le(int x) { return "x <= " + x; }
    private String ge(int x) { return "x >= " + x; }
    private String range(int a, int b) { return ge(a) + " && " + le(b); }

    private PrintWriter openTestFile(String desc) throws IOException {
        int testNo = ++nTests;
        if (includeTests == null) {
            // generate lst file when writing all tests
            try (PrintWriter lstOut = new PrintWriter(new FileWriter(LST_FILE, true))) {
                lstOut.printf(TEST_NAME_FORMAT + " %s%n", nTests, desc);
            }
        }
        if (includeTests == null || includeTests.contains(testNo)) {
            File file = new File(TESTS_DIR, String.format(TEST_NAME_FORMAT, testNo));
            PrintWriter out = new PrintWriter(file);
            System.out.printf("Writing test " + TEST_NAME_FORMAT + ": %s%n", testNo, desc);
            return out;
        } else {
            return new MockPrintWriter(); // don't write to disk
        }
    }

    private void write(String desc, String... lines) throws IOException {
        try (PrintWriter out = openTestFile(desc)) {
            for (int i = 0; i < lines.length; i++) {
                out.println(lines[i] + (i < lines.length - 1 ? " ||" : ""));
            }
        }
    }

    enum Mode { ONE, SHUFFLED, WITH_SHUFFLED }

    private void write(String desc, Mode mode, String... lines) throws IOException {
        switch (mode) {
            case ONE:
                write(desc, lines);
                break;
            case SHUFFLED:
                List<String> shuffled = new ArrayList<>(Arrays.asList(lines));
                Collections.shuffle(shuffled, RND);
                write(desc + " shuffled", shuffled.toArray(new String[0]));
                break;
            case WITH_SHUFFLED:
                write(desc, lines);
                write(desc, Mode.SHUFFLED, lines);
                break;
        }
    }

    private void writeRandom(String desc, int n, int min, int max, double leProb, double gtProb, double minProb, double maxProb) throws IOException {
        write(String.format("%s - rnd %d*{%s,%s},leProb=%.2f,gtProb=%.2f,minProb=%.2f,maxProb=%.2f", desc, n,
                min == MIN ? "min" : "" + min,
                max == MAX ? "max" : "" + max,
                leProb, gtProb, minProb, maxProb),
                streamRandomLines(n, min, max, leProb, gtProb, minProb, maxProb).toArray(String[]::new));
    }

    private Stream<String> streamRandomLines(int n, int min, int max, double leProb, double gtProb, double minProb, double maxProb) {
        return IntStream.range(0, n).mapToObj(i -> {
            double r = RND.nextDouble();
            if (r < leProb) return le(rndNum(min, max, minProb, maxProb));
            if (r < leProb + gtProb) return ge(rndNum(min, max, minProb, maxProb));
            return range(rndNum(min, max, minProb, maxProb), rndNum(min, max, minProb, maxProb));
        });
    }

    private int rndNum(int min, int max, double minProb, double maxProb) {
        double r = RND.nextDouble();
        if (r < minProb) return min;
        if (r < minProb + maxProb) return max;
        return min + RND.nextInt(max - min + 1);
    }


    private static class MockPrintWriter extends PrintWriter {
        public MockPrintWriter() { super(new StringWriter()); }
    }
}
