import java.io.*
import java.util.*
import kotlin.system.measureTimeMillis

/**
 * Solution for NEERC'2016 Problem H: Hard Refactoring
 * This solution requires correctness of the input.
 * @author Roman Elizarov
 */

val INPUT = "hard.in"
val OUTPUT = "hard.out"

fun main(args: Array<String>) {
    val time = measureTimeMillis {
        Scanner(File(INPUT)).apply { useLocale(Locale.US) }.use { it.read() }
        solve()
        PrintWriter(OUTPUT).use { it.write() }
    }
    println("Done in $time ms")
}

// input

enum class Op(val s: String) { GE(">="), LE("<=");
    override fun toString(): String = s
}

interface Case

data class Cond(val op: Op, val x: Int) : Case {
    override fun toString(): String = "x $op $x"
}

data class Range(val first: Cond, val second: Cond) : Case {
    override fun toString(): String = "$first && $second"
}

data class CaseEOF(val c: Case, val eof: Boolean)

fun Scanner.nextCond() : Cond {
    require(next() == "x") { "Expected 'x' " }
    val op = when (next()) {
        ">=" -> Op.GE
        "<=" -> Op.LE
        else -> fail("Expected '>=' or '<='")
    }
    val x = nextInt()
    require(x in min..max) { "Out of range x $op $x"}
    return Cond(op, x)
}

fun Scanner.nextCase() : CaseEOF {
    val first = nextCond()
    if (!hasNextOnLine()) return CaseEOF(first, true)
    val sep = next()
    if (sep == "||") return CaseEOF(first, false)
    require(sep == "&&")
    require(first.op == Op.GE)
    val second = nextCond()
    require(second.op == Op.LE)
    val range = Range(first, second)
    if (!hasNextOnLine()) return CaseEOF(range, true)
    require(next() == "||")
    return CaseEOF(range, false)
}

val cases = mutableListOf<Case>()

fun Scanner.read() {
    while (true) {
        val (c, eof) = nextCase()
        nextLine()
        cases.add(c)
        if (eof) break
    }
}

// solution

const val min = Short.MIN_VALUE.toInt()
const val max = Short.MAX_VALUE.toInt()

data class Item(val x: Int, val d: Int)

val result = mutableListOf<Case>()

fun addResult(l: Int, r: Int) {
    result.add(when {
        l == min -> Cond(Op.LE, r)
        r == max -> Cond(Op.GE, l)
        else -> Range(Cond(Op.GE, l), Cond(Op.LE, r))
    })
}

fun solve() {
    var cnt = 0
    var started = false
    var left = 0
    var right = 0
    cases.map { c ->
            if (c is Cond) {
                    when (c.op) {
                        Op.LE -> Range(Cond(Op.GE, min), Cond(Op.LE, c.x))
                        Op.GE -> Range(Cond(Op.GE, c.x), Cond(Op.LE, max))
                    }
                } else c as Range
            }
            .filter { r -> r.first.x <= r.second.x }
            .flatMap { r ->
                listOf(Item(r.first.x, 1), Item(r.second.x, -1))
            }
            .sortedBy { it.x }
            .forEach {
                if (cnt == 0) {
                    if (started) {
                        if (it.x > right + 1) {
                            addResult(left, right)
                            left = it.x
                        }
                    } else {
                        started = true
                        left = it.x
                    }
                }
                cnt += it.d
                if (cnt == 0) right = it.x
            }
    if (started) addResult(left, right)
}

// result

fun PrintWriter.write() {
    when {
        result.isEmpty() -> println("false")
        result.size == 1 && result.single() == Cond(Op.LE, max) -> println("true")
        else -> result.forEachIndexed { i, c ->
                print(c)
                if (i < result.size - 1) print(" ||")
                println()
            }
    }
}

//----------------- utility ------------------

fun fail(s: String): Nothing = throw IllegalArgumentException(s)

//----------------- just for validation ------------------

/**
 * Strict scanner to verify 100% correspondence between input files and input file format specification.
 * It is a drop-in replacement for [java.util.Scanner] that could be added to a solution source
 * (cut-and-paste) without breaking its ability to work with [java.util.Scanner].
 */
class Scanner(source: File) : Closeable {
    private val reader = BufferedReader(FileReader(source))
    private var line: String? = ""
    private var pos: Int = 0
    private var lineNo: Int = 0
    private var localeset: Boolean = false

    init { nextLine() }

    override fun close() {
        require(line == null) { "Extra data at the end of file" }
        try {
            reader.close()
        } catch (e: IOException) {
            fail("Failed to close with " + e)
        }

    }

    fun nextLine() {
        require(line != null) { "EOF" }
        require(pos == line!!.length) { "Extra characters on line " + lineNo }
        try {
            line = reader.readLine()
        } catch (e: IOException) {
            fail("Failed to read line with " + e)
        }
        pos = 0
        lineNo++
    }

    fun hasNextOnLine(): Boolean = pos < line!!.length

    operator fun next(): String {
        require(line != null) { "EOF" }
        require(line!!.isNotEmpty()) { "Empty line " + lineNo }
        if (pos == 0)
            require(line!![0] > ' ') { "Line $lineNo starts with whitespace" }
        else {
            require(pos < line!!.length) { "Line $lineNo is over" }
            require(line!![pos] == ' ') { "Wrong whitespace on line " + lineNo }
            pos++
            require(pos < line!!.length) { "Line $lineNo is over" }
            require(line!![pos] > ' ') { "Line $lineNo has double whitespace" }
        }
        val sb = StringBuilder()
        while (pos < line!!.length && line!![pos] > ' ')
            sb.append(line!![pos++])
        return sb.toString()
    }

    fun nextInt(): Int {
        val s = next()
        require(s.length == 1 || s[0] != '0') { "Extra leading zero in number $s on line $lineNo" }
        require(s[0] != '+') { "Extra leading '+' in number $s on line $lineNo" }
        try {
            return Integer.parseInt(s)
        } catch (e: NumberFormatException) {
            fail("Malformed number $s on line $lineNo")
        }

    }

    fun useLocale(locale: Locale) {
        require(locale === Locale.US)
        localeset = true
    }
}
