#ifndef __invoke_h__
#define __invoke_h__

#include <windows.h>

#define int64 long long

/**
 * Invocation request. Contains all the information needed for the invocation.
 */
typedef struct {

    /**
     * Name of the program to invoke. Cannot be NULL.
     */
    char* program;

    /**
     * The number of program arguments.
     */
    int argc;

    /**
     * Program arguments, can be NULL if argc = 0.
     */
    char** argv;

    /**
     * Directory where the invocation must be performed.
     * Can be NULL, in this case current directory of the parent
     * process is used (default in Windows API).
     */
    char* directory;

    /**
     * Account name, if the invocation under alternative user account is
     * to be performed. Can be NULL, in this case invocation is
     * performed under current user account
     */
    char* login;

    /**
     * Password for the account. Can be NULL if name is NULL.
     */
    char* password;

    /**
     * Time limit for the invocation, in milliseconds. If timeLimit = 0,
     * no time limit is used.
     */
    int64 timeLimit;

    /**
     * Memory limit for the invocation in bytes. If memoryLimit = 0,
     * no memory limit is used.
     */
    int64 memoryLimit;

    /**
     * Idleness limit for the invocation in milliseconds. If idlenessLimit = 0,
     * no idleness limit is used.
     */
    int64 idlenessLimit;

    /**
     * The minimal load of the processor that is considered non-idle. Its
     * value must be between 0 and 1 inclusive.
     */
    double requiredProcessorLoad;

    /**
     * Additional flags that must be used for the invocation. The actual
     * value of the flags field must be logical OR of the flags to be used.
     *
     * Possible flags are:
     *   INVOKE_REDIRECT_STD_STREAMS
     *       If this flag is set, the standard input/output/error streams
     *       are redirected to/from files with names specified by inputFileName,
     *       outputFileName and errorFileName fields.
     *   INVOKE_ALLOW_CREATE_PROCESSES
     *       If this flag is set, the process is allowed to create new processes.
     *       By default it causes SECURITY_VIOLATION outcome.
     *   INVOKE_IGNORE_FIRST_CHANCE_EXCEPTIONS
     *       If this flag is set, the first chance exceptions are ignored and
     *       returned back to the created process. By default the cause
     *       CRASH outcome. Second chance exceptions always cause CRASH outcome.
     *   INVOKE_SECURITY_VIOLATION_IF_FAILED_TO_INVOKE
     *       If this flag is set, the failure to create process because of
     *       windows error causes SECURITY_VIOLATION outcome, not FAIL.
     *   INVOKE_SHOW_APPLICATION_WINDOW
     *       If this flag is set, the application window will be shown on
     *       the current desktop.
     */
    int flags;

    /**
     * Reserved for future use. Must be zero.
     */
    int reserved;

    /**
     * The name of the file to redirect standard input from. If REDIRECT_STD_STREAMS
     * flag is not set, this value is ignored.
     *
     * Can be NULL, in this case handle to "nul" is passed to the called program.
     */
    char* inputFileName;

    /**
     * The name of the file to redirect standard output to. If REDIRECT_STD_STREAMS
     * flag is not set, this value is ignored.
     *
     * Can be NULL, in this case handle to "nul" is passed to the called program.
     */
    char* outputFileName;

    /**
     * The name of the file to redirect standard error to. If REDIRECT_STD_STREAMS
     * flag is not set, this value is ignored.
     *
     * Can be NULL, in this case handle to "nul" is passed to the called program.
     */
    char* errorFileName;

    /**
     * Pointer to an environment block for the new process. If this parameter
     * is NULL, the new process uses current environment.
     *
     * An environment block consists of a null-terminated block of
     * null-terminated strings. Each string is in the form:
     * name=value
     *
     * Note that an environment block is terminated by two zero bytes:
     * one for the last string, one more to terminate the block.
     */
    char* environment;

    /**
     * Pointer to arbitrary information, this pointer is passed as a paramter
     * to the callback routine if any. Can be NULL.
     */
    void* userInfo;
} InvocationRequest;

/**
 * Allow to create processes invocation flag.
 *
 * If this flag is set, the process is allowed to create new processes.
 * By default it causes SECURITY_VIOLATION outcome.
 */
const int INVOKE_ALLOW_CREATE_PROCESSES = 0x00000001;

/**
 * Some compilers, for example Borland Delphi, produce executables that are incapable
 * of handling their own exceptions. In such case ignoring first chance exceptions
 * (default behavior) cases the program to hang. You can set this flag to terminate
 * program even on first chance exception.
 *
 * If this flag is not set, the first chance exceptions are ignored and
 * returned back to the created process. If this flag is set they cause
 * CRASH outcome. Second chance exceptions always cause CRASH outcome.
 */
const int INVOKE_TERMINATE_ON_FIRST_CHANCE_EXCEPTIONS = 0x00000002;

/**
 * Ignore first chance exceptions invocation flag.
 *
 * If this flag is set, the failure to create process because of
 * windows error causes SECURITY_VIOLATION outcome, not FAIL.
 */
const int INVOKE_SECURITY_VIOLATION_IF_FAILED_TO_INVOKE = 0x00000004;

/**
 * Show application window flag.
 *
 * If this flag is set, the application window will be shown on
 * the current desktop.
 */
const int INVOKE_SHOW_APPLICATION_WINDOW = 0x00000008;

/**
 * Set single affinity flag.
 *
 * If this flag is set, the application is forced to be run on the first
 * CPU/CPU core only.
 */
const int INVOKE_SET_SINGLE_AFFINITY = 0x00000010;


/**
 * Invocation outcome.
 */
typedef enum {
    /**
     * Invocation failed to complete successfully.
     */
    FAIL,

    /**
     * Program successfully terminated within the given constraints.
     */
    SUCCESS,

    /**
     * Program attempts to violate security restrictions, for example,
     * by trying to create the child process.
     *
     * This outcome is also returned if  INVOKE_SECURITY_VIOLATION_IF_FAILED_TO_INVOKE
     * flag is set and the program cannot be executed because of windows error.
     */
    SECURITY_VIOLATION,

    /**
     * Program exceeded its time limit.
     */
    TIME_LIMIT_EXCEEDED,

    /**
     * Program exceeded its memory limit.
     */
    MEMORY_LIMIT_EXCEEDED,

    /**
     * Program exceeded its idleness limit.
     */
    IDLENESS_LIMIT_EXCEEDED,

    /**
     * Program produced the uncaught exception.
     */
    CRASH,

    /**
     * Program was terminated because the callback routine recommended to
     * terminate the program.
     */
    USER_TERMINATED
} Outcome;

/**
 * Result of the invocation.
 */
typedef struct {
    /**
     * Whether the program was forced to terminate.
     */
    bool forcedToTerminate;

    /**
     * Outcome of the invocation
     */
    Outcome outcome;

    /**
     * Program exit code. This value is meaningless if forcedToTerminate is true.
     */
    DWORD exitCode;

    /**
     * The code of the uncaught exception thrown by the program.
     * This value if meaningful only if outcome = CRASH.
     */
    unsigned long exceptionCode;

    /**
     * Processor time consumed by the program.
     */
    int64 timeConsumed;

    /**
     * System time passed while the program was invoked.
     */
    int64 timePassed;

    /**
     * Peak memory consumed by the program.
     */
    int64 peakMemoryConsumed;

    /**
     * Human readable comment. Invocation routine may choose to put here
     * some human readable information about invocation. The calling
     * routine must free strlen(comment) + 1 bytes of memory after
     * examining the comment. Can be NULL.
     */
    char* comment;
} InvocationResult;

/**
 * Invoked process state.
 */
typedef enum {
    /**
     * Invocation routine is preparing for the invocation.
     */
    PREPARING,

    /**
     * Invocation routine failed to invoke the specified program.
     */
    FAILED_TO_INVOKE,

    /**
     * The invoked program is running.
     */
    RUNNING,

    /**
     * The invoked program has crashed.
     */
    CRASHED,

    /**
     * The invoked program is idle.
     */
    IDLE,

    /**
     * The invoked program is beeing terminated.
     */
    TERMINATING,

    /**
     * The invoked program is terminated.
     */
    TERMINATED
} ProcessState;

/**
 * Current status of the invocation.
 */
typedef struct {
    /**
     * Invoked process state.
     */
    ProcessState state;

    /**
     * Processor time consumed by the program.
     */
    int64 timeConsumed;

    /**
     * System time passed while the program was invoked.
     */
    int64 timePassed;

    /**
     * Current memory consumed by the program.
     */
    int64 currentMemoryConsumed;

    /**
     * Peak memory consumed by the program.
     */
    int64 peakMemoryConsumed;

    /**
     * Time period that the program has been idle for.
     */
    int64 idlenessPeriod;

    /**
     * Current processor load with the invoked task.
     */
    double processorLoad;

    /**
     * Pointer that was passed to the invocation routine.
     */
    void* userInfo;
} InvocationStatus;

/**
 * Callback routine for the invocation
 *
 * @param status - Current status of the invocation.
 * @return true if invocation must be terminated.
 */
typedef bool (*InvocationCallback)(InvocationStatus* status);

/**
 * Invocation error.
 *
 * One of the parameters that must not have been NULL is NULL.
 */
const int ERROR_INVOKE_NULL_PARAMETER               = 0x20000001;

/**
 * Invocation error.
 *
 * One of the invocation parameters is invalid.
 */
const int ERROR_INVOKE_INVALID_INVOCATION_PARAMETER = 0x20000002;

/**
 * Invoke external process with the given constraints.
 *
 * @param invocationRequest     - Request for the invocation.
 * @param invocationResult      - Structure to be filled with invocation result.
 * @param invocationCallback    - Callback routine.
 */
extern "C"
__declspec(dllexport)
void invoke(
    InvocationRequest* invocationRequest,
    InvocationResult* invocationResult,
    InvocationCallback callback
);


typedef struct {
    HANDLE input;
    HANDLE output;
    HANDLE error;
} StandardHandles;
extern "C"
__declspec(dllexport)
void invokeWithHandles(
    InvocationRequest* invocationRequest,
    InvocationResult* invocationResult,
    InvocationCallback callback,
    StandardHandles* standardHandles
);
#endif